@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 #!-sb-fluid (declaim (inline store-word))
 (defun store-word (word base &optional (offset 0) (lowtag 0))
@@ -51,20 +51,12 @@
   system-area-pointer)
 (define-alien-variable ("fixedobj_free_pointer" *fixedobj-space-free-pointer*)
   system-area-pointer)
-(defun immobile-space-addr-p (addr)
-  (declare (type word addr))
-  (or (let ((start fixedobj-space-start))
-        (<= start addr (truly-the word (+ start (1- fixedobj-space-size)))))
-      (let ((start varyobj-space-start))
-        (<= start addr (truly-the word (+ start (1- varyobj-space-size)))))))
-(defun immobile-space-obj-p (obj)
-  (immobile-space-addr-p (get-lisp-obj-address obj)))
 
-(define-load-time-global *immobile-space-mutex*
-    (sb!thread:make-mutex :name "Immobile space"))
+(!define-load-time-global *immobile-space-mutex*
+    #.(sb-thread:make-mutex :name "Immobile space"))
 
 (eval-when (:compile-toplevel)
-  (assert (eql code-code-size-slot 1))
+  (assert (eql code-boxed-size-slot 1))
   (assert (eql code-debug-info-slot 2)))
 
 (define-alien-variable "varyobj_holes" long)
@@ -112,16 +104,12 @@
 
 (declaim (inline hole-size))
 (defun hole-size (hole-address) ; in bytes
-  (the (and fixnum unsigned-byte)
-    (sap-ref-lispobj (int-sap hole-address) (ash code-code-size-slot word-shift))))
+  (ash (sap-ref-32 (int-sap hole-address) 4) word-shift))
 
 (declaim (inline (setf hole-size)))
 (defun (setf hole-size) (new-size hole) ; NEW-SIZE is in bytes
-  ;; FIXME: (SETF SET-REF-LISPOBJ) does not accept a fixnum
-  ;; because 'any-reg' is not an allowed storage class.
-  (setf (sap-ref-word (int-sap hole) (ash code-code-size-slot word-shift))
-        ;; 0 boxed words, so %CODE-CODE-SIZE covers everything.
-        (ash new-size n-fixnum-tag-bits)))
+  (setf (sap-ref-32 (int-sap hole) 4) (ash new-size (- word-shift)))
+  new-size)
 
 (declaim (inline hole-end-address))
 (defun hole-end-address (hole-address)
@@ -273,12 +261,12 @@
       (setf (hole-size hole) (- hole-end hole))))
   (add-to-freelist hole))
 
-(defun allocate-immobile-bytes (n-bytes word0 word1 lowtag errorp)
+(defun allocate-immobile-obj (n-bytes word0 word1 lowtag errorp)
   (declare (type (and fixnum unsigned-byte) n-bytes))
   (setq n-bytes (align-up n-bytes (* 2 n-word-bytes)))
   ;; Can't allocate fewer than 4 words due to min hole size.
   (aver (>= n-bytes (* 4 n-word-bytes)))
-  (sb!thread::with-system-mutex (*immobile-space-mutex* :without-gcing t)
+  (sb-thread::with-system-mutex (*immobile-space-mutex* :without-gcing t)
    (unless (zerop varyobj-holes)
      ;; If deferred sweep needs to happen, do so now.
      ;; Concurrency could potentially be improved here: at most one thread
@@ -294,32 +282,34 @@
                 nil-value)
           (unallocate hole-addr)
           (if (eql (setq hole-addr next) 0) (return))))))
-   (let ((addr
-          (or (and *immobile-freelist*
-                   (or (find-in-freelist n-bytes '=) ; 1. Exact match?
-                       ;; 2. Try splitting a hole, adding some slack so that
-                       ;;    both pieces can potentially be used.
-                       (let ((found (find-in-freelist (+ n-bytes 192) '<=)))
-                         (when found
-                           (let* ((actual-size (hole-size found))
-                                  (remaining (- actual-size n-bytes)))
-                             (aver (not (logtest actual-size lowtag-mask)))
-                             (setf (hole-size found) remaining) ; Shorten the lower piece
-                             (add-to-freelist found)
-                             (+ found remaining)))))) ; Consume the upper piece
-              ;; 3. Extend the frontier.
-              (let* ((addr (sap-int *varyobj-space-free-pointer*))
-                     (free-ptr (+ addr n-bytes))
-                     (limit (+ varyobj-space-start varyobj-space-size)))
-                (when (> free-ptr limit)
-                  (cond (errorp
-                         (format t "~&Immobile space exhausted~%")
-                         (sb!debug:print-backtrace)
-                         (sb!impl::%halt))
-                        (t
-                         (return-from allocate-immobile-bytes 0))))
-                (set-varyobj-space-free-pointer free-ptr)
-                addr))))
+   (let* ((residual)
+          (shrunk-size)
+          (addr
+           (or (and *immobile-freelist*
+                    (or (find-in-freelist n-bytes '=) ; 1. Exact match?
+                        ;; 2. Try splitting a hole, adding some slack so that
+                        ;;    both pieces can potentially be used.
+                        (let ((found (find-in-freelist (+ n-bytes 192) '<=)))
+                          (when found
+                            (let* ((actual-size (hole-size found))
+                                   (remaining (- actual-size n-bytes)))
+                              (aver (not (logtest actual-size lowtag-mask)))
+                              (setq residual found ; Shorten the lower piece
+                                    shrunk-size remaining)
+                              (+ found remaining)))))) ; Consume the upper piece
+               ;; 3. Extend the frontier.
+               (let* ((addr (sap-int *varyobj-space-free-pointer*))
+                      (free-ptr (+ addr n-bytes))
+                      (limit (+ varyobj-space-start varyobj-space-size)))
+                 (when (> free-ptr limit)
+                   (cond (errorp
+                          (format t "~&Immobile space exhausted~%")
+                          (sb-debug:print-backtrace)
+                          (sb-impl::%halt))
+                         (t
+                          (return-from allocate-immobile-obj nil))))
+                 (set-varyobj-space-free-pointer free-ptr)
+                 addr))))
      (aver (not (logtest addr lowtag-mask))) ; Assert proper alignment
      ;; Compute the start and end of the first page consumed.
      (let* ((page-start (logandc2 addr (1- immobile-card-bytes)))
@@ -354,14 +344,28 @@
      (#!+64-bit system-area-ub64-fill
       #!-64-bit system-area-ub32-fill
       0 (int-sap addr) 2 (- (ash n-bytes (- word-shift)) 2))
+     ;; Only after making the new object can we reduce the size of the hole
+     ;; that contained the new allocation (if it entailed chopping a hole
+     ;; into parts). In this way, heap scans do not read junk.
+     (when residual
+       (setf (hole-size residual) shrunk-size)
+       (add-to-freelist residual))
+     ;; The object is live despite not having a tagged pointer yet nor
+     ;; this code being pseudoatomic, because the mutex acquire has
+     ;; :WITHOUT-GCING. Ideally we'd have some notion of hazard pointers
+     ;; that could prevent GC from evicting objects from pointed-to pages
+     ;; so that we needn't inhibit GC.
      (%make-lisp-obj (logior addr lowtag)))))
 
+;;; I don't know whether immobile space vectors actually work.
+;;; I think the idea was to use them as fd-stream buffers when applicable
+;;; instead of having a finalizer on OS-allocated memory.
 (defun allocate-immobile-vector (widetag length words)
-  (allocate-immobile-bytes (pad-data-block (+ words vector-data-offset))
-                           widetag
-                           (fixnumize length)
-                           other-pointer-lowtag
-                           t))
+  (allocate-immobile-obj (pad-data-block (+ words vector-data-offset))
+                         widetag
+                         (fixnumize length)
+                         other-pointer-lowtag
+                         t))
 
 (defun allocate-immobile-simple-vector (n-elements)
   (allocate-immobile-vector simple-vector-widetag n-elements n-elements))
@@ -378,8 +382,7 @@
 
 (defun alloc-immobile-symbol ()
   (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag symbol-size
-                      (logior (ash (1- symbol-size) n-widetag-bits) symbol-widetag)
-                      nil)))
+                      (logior (ash (1- symbol-size) n-widetag-bits) symbol-widetag))))
 (defun make-immobile-symbol (name)
   (let ((symbol (truly-the symbol (alloc-immobile-symbol))))
     ;; no pin, it's immobile (and obviously live)
@@ -394,46 +397,63 @@
 
 (defun alloc-immobile-fdefn ()
   (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag fdefn-size
-                      (logior (ash (1- fdefn-size) n-widetag-bits) fdefn-widetag)
-                      nil)))
+                      (logior (ash (1- fdefn-size) n-widetag-bits) fdefn-widetag))))
 
 #!+immobile-code
 (progn
 (defun alloc-immobile-gf ()
   (values (%primitive alloc-immobile-fixedobj fun-pointer-lowtag 6 ; kludge
-                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag)
-                      nil)))
+                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag))))
 (defun make-immobile-gf (layout slot-vector)
   (let ((gf (truly-the funcallable-instance (alloc-immobile-gf))))
     ;; Set layout prior to writing raw slots
     (setf (%funcallable-instance-layout gf) layout)
-    (setf (sap-ref-word (int-sap (get-lisp-obj-address gf))
-                        (- (ash funcallable-instance-trampoline-slot word-shift)
-                           fun-pointer-lowtag))
-          (truly-the word
-           (+ (get-lisp-obj-address gf) (- fun-pointer-lowtag) (ash 4 word-shift))))
+    ;; just being pedantic - liveness is preserved by the stack reference,
+    ;; and address is fixed, by definition.
+    (with-pinned-objects (gf)
+      (let ((addr (logandc2 (get-lisp-obj-address gf) lowtag-mask)))
+        (setf (sap-ref-word (int-sap addr)
+                            (ash funcallable-instance-trampoline-slot word-shift))
+              (truly-the word (+ addr (ash 4 word-shift))))))
     (%set-funcallable-instance-info gf 0 slot-vector)
     (!set-fin-trampoline gf)
     gf))
 
 (defun alloc-immobile-trampoline ()
-  (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag 6
-                      #.(make-code-header-word 4) ; boxed word count
-                      (ash (* 2 n-word-bytes) n-fixnum-tag-bits))))
+  (let ((obj (%primitive alloc-immobile-fixedobj other-pointer-lowtag 6
+                         ;; total word count
+                         (logior (ash 6 code-header-size-shift) code-header-widetag))))
+    (with-pinned-objects (obj)
+      (setf (sap-ref-word (int-sap (get-lisp-obj-address obj))
+                          (- (ash code-boxed-size-slot word-shift)
+                             other-pointer-lowtag))
+            ;; The 'fixups' slot permanently contains 0. Therefore we can say that
+            ;; there are only 3 boxed header words instead of the usual 4 words.
+            (* 3 n-word-bytes))) ; boxed size in bytes, untagged
+    obj))
 
-;;; Test whether there is room to allocate NBYTES.
-;;; This only looks at the space in the frontier, not the free space list.
-(defun immobile-code-free-space-check (nbytes)
-  (let ((avail (- (+ varyobj-space-start varyobj-space-size)
-                  (sap-int *varyobj-space-free-pointer*))))
-    (>= avail nbytes)))
 ) ; end PROGN
 ) ; end PROGN
 
-;;; Enforce limit on boxed words dependent on how many bits it gets in header.
-;;; Enforce limit on unboxed size. 22 bits = 4MiB, quite generous for code.
-(declaim (ftype (sfunction (t (unsigned-byte #!+64-bit 32 #!-64-bit 22)
-                              (unsigned-byte 22))
+(declaim (inline immobile-space-addr-p))
+(defun immobile-space-addr-p (addr)
+  (declare (type word addr) (ignorable addr))
+  #!+immobile-space
+  (or (let ((start fixedobj-space-start))
+        (<= start addr (truly-the word (+ start (1- fixedobj-space-size)))))
+      (let ((start varyobj-space-start))
+        (<= start addr (truly-the word (+ start (1- varyobj-space-size)))))))
+
+(defun immobile-space-obj-p (obj)
+  (immobile-space-addr-p (get-lisp-obj-address obj)))
+
+;;; Enforce limit on boxed words based on maximum total number of words
+;;; that can be indicated in the header for 32-bit words.
+;;; 22 bits = 4MiB, quite generous for one code object.
+;;; 25 bits is the maximum unboxed size expressed in bytes,
+;;; if n-word-bytes = 8 and almost the entire code object is unboxed
+;;; which is, practically speaking, not really possible.
+(declaim (ftype (sfunction (t (unsigned-byte 22) (unsigned-byte 25))
                            code-component)
                 allocate-code-object))
 ;;; Allocate a code component with BOXED words in the header
@@ -442,40 +462,61 @@
 ;;; are made for alignment considerations or the fixed slots.
 (defun allocate-code-object (space boxed unboxed)
   (declare (ignorable space))
-  #!+gencgc
-  ;; This uses ATOMIC-INCF to get automatic wraparound, not because atomicity
-  ;; makes things deterministic per se. If multiple threads are allocating
-  ;; code objects, the order is unpredictable.
-  (let ((serialno (atomic-incf sb!fasl::*code-serialno*)))
-    ;; FIXME: remove this use of WITHOUT-GCing and use pseudo-atomic.
-    (without-gcing
-      (block nil
-        #!+immobile-code
-        (when (member space '(:immobile :auto))
-          (let ((code (allocate-immobile-bytes
-                          (align-up (+ (* boxed n-word-bytes) unboxed)
-                                    (* 2 n-word-bytes))
-                          (make-code-header-word boxed)
-                          (logior (logand (ash serialno 32) most-positive-word)
-                                  (ash unboxed n-fixnum-tag-bits))
-                          other-pointer-lowtag
-                          (eq space :immobile))))
-            (unless (eql code 0)
-              (setf (%code-debug-info code) nil)
-              (return code))
-            #+nil (warn "immobile code space is too small")))
-        (%make-lisp-obj
-         (alien-funcall (extern-alien "alloc_code_object"
-                                      (function unsigned (unsigned 32)
-                                                (unsigned 32) (unsigned 32)))
-                        boxed unboxed serialno)))))
-  #!-gencgc
-  (%primitive allocate-code-object boxed unboxed))
-
-#!+64-bit
-(progn
-  (declaim (inline %code-code-size))
-  (defun %code-serialno (code) ; return high 4 bytes
-    (ash (%%code-code-size code) (- n-fixnum-tag-bits 32)))
-  (defun %code-code-size (code) ; clip to max imposed by allocate-code-object
-    (ldb (byte 22 0) (%%code-code-size code))))
+  (let* ((total-words
+          (the (unsigned-byte 22) ; Enforce limit on total words as well
+               (align-up (+ boxed (ceiling unboxed n-word-bytes)) 2)))
+         (code
+          #!+gencgc
+          (or #!+immobile-code
+              (when (member space '(:immobile :auto))
+                ;; We don't need to inhibit GC here - ALLOCATE-IMMOBILE-OBJ does it.
+                ;; Indicate that there are initially 2 boxed words, otherwise
+                ;; immobile space GC thinks this object is freeable.
+                (allocate-immobile-obj (ash total-words word-shift)
+                                       (logior (ash total-words code-header-size-shift)
+                                               code-header-widetag)
+                                       (ash 2 n-fixnum-tag-bits)
+                                       other-pointer-lowtag
+                                       (eq space :immobile)))
+              ;;; x86-64 has a vop which is nothing more than wrapping
+              ;; pseudo-atomic around a call to alloc_code_object() in the C runtime.
+              ;; The vop is defined in such a way that it can't be inserted into
+              ;; this fuction, but instead needs an out-of-line call to a helper function
+              ;; (because it clobbers all registers and doesn't indicate that)
+              #!+(and x86-64 (not win32))
+              (alloc-dynamic-space-code total-words)
+              #!-(and x86-64 (not win32))
+              (without-gcing
+               (%make-lisp-obj
+                (alien-funcall (extern-alien "alloc_code_object"
+                                             (function unsigned (unsigned 32)))
+                               total-words))))
+          #!+cheneygc
+          (%primitive var-alloc total-words 'alloc-code
+                      ;; subtract 1 because var-alloc always adds 1 word
+                      ;; for the header, which is not right for code objects.
+                      -1 code-header-widetag other-pointer-lowtag)))
+    ;; The 1st slot beyond the header stores the boxed header size in bytes
+    ;; as an untagged number, which has the same representation as a tagged
+    ;; value denoting a word count if WORD-SHIFT = N-FIXNUM-TAG-BITS.
+    ;; This slot is allowed to be 0 prior to writing any pointer descriptors
+    ;; into the object.
+    ;; Use ATOMIC-INCF on the serialno to get automatic wraparound,
+    ;; and not because atomicity makes things deterministic, which it doesn't
+    ;; if there are several threads allocating code.
+    ;; TODO: this unnecessarily calls CODE-HEADER-SET if code cards use soft
+    ;; marking. Maybe pin the object and use (SETF SAP-REF-WORD) instead.
+    (setf (code-header-ref code code-boxed-size-slot)
+          (%make-lisp-obj
+           (logior (ash boxed word-shift)
+                   #!+64-bit
+                   (logand (ash (atomic-incf sb-fasl::*code-serialno*) 32)
+                           most-positive-word))))
+    ;; Is this slot assignment necessary?  Both the C and Lisp backtrace logic
+    ;; check whether debug-info is an INSTANCE before operating with the value,
+    ;; and GC doesn't care what it is. Since stores to the code go through an
+    ;; assembly routine if soft marking is enabled, it's a spurious call
+    ;; to write a value that needn't add this object to a remembered set.
+    ;; FIXME: find out what breaks if this is removed.
+    (setf (%code-debug-info code) nil)
+    code))
