@@ -9,9 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
-
-(/show0 "entering backq.lisp")
+(in-package "SB-IMPL")
 
 ;; An unquoting COMMA struct.
 (defstruct (comma (:constructor unquote (expr &optional (kind 0)))
@@ -40,8 +38,6 @@
 (defvar *backquote-depth* 0 "how deep we are into backquotes")
 (defvar *bq-error* "Comma not inside a backquote.")
 
-(/show0 "backq.lisp 50")
-
 ;;; the actual character macro
 (defun backquote-charmacro (stream char)
   (declare (ignore char))
@@ -54,8 +50,6 @@
          stream "~S is not a well-formed backquote expression" result)
         result)))
 
-(/show0 "backq.lisp 64")
-
 (defun comma-charmacro (stream char)
   (declare (ignore char))
   (declare (notinline read-char unread-char))
@@ -76,8 +70,6 @@
     (unquote (let ((*backquote-depth* (1- *backquote-depth*)))
                (read stream t nil t)) flag)))
 
-(/show0 "backq.lisp 83")
-
 ;; KLUDGE: 'sfunction' is not a defined type yet.
 (declaim (ftype (function (t fixnum boolean) (values t t &optional))
                 qq-template-to-sexpr qq-template-1))
@@ -140,8 +132,6 @@
                          subexpr)
                    operator)))))
 
-(/show0 "backq.lisp 139")
-
 ;; Find the longest suffix comprised wholly of self-evaluating and/or quoted
 ;; SUBFORMS. DOTTED-P indicates that the last item represents what was in the
 ;; CDR of the last cons of the original list. Return the modified SUBFORMS
@@ -335,5 +325,3 @@
 #+sb-xc-host ; proper definition happens for the target
 (defun simple-reader-error (stream format-string &rest format-args)
   (error "READER-ERROR on stream ~S: ~?" stream format-string format-args))
-
-(/show0 "done with backq.lisp")
