@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; putting ourselves out of our misery when things become too much to bear
 
@@ -20,6 +20,14 @@
   (%primitive print "too early in cold init to recover from errors")
   (%halt))
 
+(defun !cold-failed-aver (expr)
+  ;; output the message and invoke ldb monitor
+  (terpri)
+  (write-line "failed AVER:")
+  (write expr)
+  (terpri)
+  (%halt))
+
 ;;; last-ditch error reporting for things which should never happen
 ;;; and which, if they do happen, are sufficiently likely to torpedo
 ;;; the normal error-handling system that we want to bypass it
@@ -32,10 +40,10 @@
 ;;;; !COLD-INIT
 
 ;;; a list of toplevel things set by GENESIS
-(defvar *!cold-toplevels*)                 ; except for DEFUNs and SETF macros
-(defvar *!cold-setf-macros*)               ; just SETF macros
-(defvar *!cold-defconstants*)              ; just DEFCONSTANT-EQXs
-(defvar *!cold-defuns*)                    ; just DEFUNs
+(defvar *!cold-toplevels*)   ; except for DEFUNs and SETF macros
+(defvar *!cold-setf-macros*) ; just SETF macros
+(defvar *!cold-defuns*)      ; just DEFUNs
+(defvar *!cold-defsymbols*)  ; "easy" DEFCONSTANTs and DEFPARAMETERs
 
 ;;; a SIMPLE-VECTOR set by GENESIS
 (defvar *!load-time-values*)
@@ -48,49 +56,21 @@
        (/primitive-print ,(symbol-name name))
        (,name)))
 
-(defun !encapsulate-stuff-for-cold-init (&aux names)
-  (flet ((encapsulate-1 (name handler)
-           (encapsulate name '!cold-init handler)
-           (push name names)))
-    (encapsulate-1 '%failed-aver
-                   (lambda (f expr)
-                     ;; output the message before signaling error,
-                     ;; as it may be this is too early in the cold init.
-                     (fresh-line)
-                     (write-line "failed AVER:")
-                     (write expr)
-                     (terpri)
-                     (funcall f expr)))
-
-    (encapsulate-1
-     'find-package
-     (lambda (f designator)
-       (cond ((packagep designator) designator)
-             (t (funcall f (let ((s (string designator)))
-                             (if (eql (mismatch s "SB!") 3)
-                                 (concatenate 'string "SB-" (subseq s 3))
-                                 s))))))))
-  names)
-
-(defmacro !with-init-wrappers (&rest forms)
-  `(let ((wrapped-functions (!encapsulate-stuff-for-cold-init)))
-     ,@forms
-     (dolist (f wrapped-functions) (unencapsulate f '!cold-init))))
-
 (defun !c-runtime-noinform-p () (/= (extern-alien "lisp_startup_options" char) 0))
 
 ;;; called when a cold system starts up
-(defun !cold-init (&aux real-choose-symbol-out-fun)
+(defun !cold-init (&aux (real-choose-symbol-out-fun #'choose-symbol-out-fun)
+                        (real-failed-aver-fun #'%failed-aver))
   "Give the world a shove and hope it spins."
 
-  #!+sb-show
-  (sb!int::cannot-/show "Test of CANNOT-/SHOW [don't worry - this is expected]")
   (/show0 "entering !COLD-INIT")
+  (setf (symbol-function '%failed-aver) #'!cold-failed-aver)
   (setq *readtable* (make-readtable)
         *print-length* 6 *print-level* 3)
   (setq *error-output* (!make-cold-stderr-stream)
                       *standard-output* *error-output*
                       *trace-output* *error-output*)
+  (/show "testing '/SHOW" *print-length* *print-level*) ; show anything
   (unless (!c-runtime-noinform-p)
     (write-string "COLD-INIT... "))
 
@@ -99,10 +79,16 @@
   ;; And be extra paranoid - ensure that it really gets called.
   (locally (declare (notinline fboundp)) (fboundp '(setf !zzzzzz)))
 
+  ;; Anyone might call RANDOM to initialize a hash value or something;
+  ;; and there's nothing which needs to be initialized in order for
+  ;; this to be initialized, so we initialize it right away.
+  ;; Indeed, INIT-INITIAL-THREAD needs a random number.
+  (show-and-call !random-cold-init)
+
   ;; Ensure that *CURRENT-THREAD* and *HANDLER-CLUSTERS* have sane values.
   ;; create_thread_struct() assigned NIL/unbound-marker respectively.
-  (sb!thread::init-initial-thread)
-  (show-and-call sb!kernel::!target-error-cold-init)
+  (sb-thread::init-initial-thread)
+  (show-and-call sb-kernel::!target-error-cold-init)
 
   ;; Putting data in a synchronized hashtable (*PACKAGE-NAMES*)
   ;; requires that the main thread be properly initialized.
@@ -115,25 +101,19 @@
   ;; that would be nice, and would tidy up some other things too.
   (show-and-call !printer-cold-init)
   ;; Because L-T-V forms have not executed, CHOOSE-SYMBOL-OUT-FUN doesn't work.
-  (setf real-choose-symbol-out-fun #'choose-symbol-out-fun)
   (setf (symbol-function 'choose-symbol-out-fun)
         (lambda (&rest args) (declare (ignore args)) #'output-preserve-symbol))
 
   ;; *RAW-SLOT-DATA* is essentially a compile-time constant
   ;; but isn't dumpable as such because it has functions in it.
-  (show-and-call sb!kernel::!raw-slot-data-init)
-
-  ;; Anyone might call RANDOM to initialize a hash value or something;
-  ;; and there's nothing which needs to be initialized in order for
-  ;; this to be initialized, so we initialize it right away.
-  (show-and-call !random-cold-init)
+  (show-and-call sb-kernel::!raw-slot-data-init)
 
   ;; Must be done before any non-opencoded array references are made.
-  (show-and-call sb!vm::!hairy-data-vector-reffer-init)
+  (show-and-call sb-vm::!hairy-data-vector-reffer-init)
 
   (show-and-call !character-database-cold-init)
   (show-and-call !character-name-database-cold-init)
-  (show-and-call sb!unicode::!unicode-properties-cold-init)
+  (show-and-call sb-unicode::!unicode-properties-cold-init)
 
   ;; All sorts of things need INFO and/or (SETF INFO).
   (/show0 "about to SHOW-AND-CALL !GLOBALDB-COLD-INIT")
@@ -143,12 +123,7 @@
   ;; the basic type machinery needs to be initialized before toplevel
   ;; forms run.
   (show-and-call !type-class-cold-init)
-  ;; cold-init-wrappers are closures. Installing a closure as a
-  ;; named function requires consing immobile space code.
-  #!+immobile-code (setq sb!vm::*immobile-space-mutex*
-                         (sb!thread:make-mutex :name "Immobile space"))
-  (!with-init-wrappers (show-and-call sb!kernel::!primordial-type-cold-init))
-  (show-and-call !world-lock-cold-init)
+  (show-and-call sb-kernel::!primordial-type-cold-init)
   (show-and-call !classes-cold-init)
   (show-and-call !early-type-cold-init)
   (show-and-call !late-type-cold-init)
@@ -164,31 +139,37 @@
   ;; Must be done before toplevel forms are invoked
   ;; because a toplevel defstruct will need to add itself
   ;; to the subclasses of STRUCTURE-OBJECT.
-  (show-and-call sb!kernel::!set-up-structure-object-class)
+  (show-and-call sb-kernel::!set-up-structure-object-class)
 
-  (dolist (x *!cold-defconstants*)
-    (destructuring-bind (name source-loc &optional docstring) x
-      (setf (info :variable :kind name) :constant)
-      (when source-loc (setf (info :source-location :constant name) source-loc))
-      (when docstring (setf (fdocumentation name 'variable) docstring))))
-  (!with-init-wrappers
-   (dolist (x *!cold-defuns*)
-     (destructuring-bind (name inline-expansion dxable-args) x
-       (%defun name (fdefinition name) inline-expansion dxable-args))))
+  ;; Genesis is able to perform some of the work of DEFCONSTANT and
+  ;; DEFPARAMETER, but not all of it. It assigns symbol values, but can not
+  ;; manipulate globaldb. Therefore, a subtlety of these macros for bootstrap
+  ;; is that we see each DEFthing twice: once during cold-load and again here.
+  ;; Now it being the case that DEFPARAMETER implies variable assignment
+  ;; unconditionally, you may think it should assign. No! This was logically
+  ;; ONE use of the defining macro, but split into pieces as a consequence
+  ;; of the implementation.
+  (dolist (x *!cold-defsymbols*)
+    (destructuring-bind (fun name source-loc . docstring) x
+      (aver (boundp name)) ; it's a bug if genesis didn't initialize
+      (ecase fun
+        (sb-c::%defconstant
+         (apply #'sb-c::%defconstant name (symbol-value name) source-loc docstring))
+        (sb-impl::%defparameter ; use %DEFVAR which will not clobber
+         (apply #'%defvar name source-loc nil docstring)))))
+  (dolist (x *!cold-defuns*)
+    (destructuring-bind (name inline-expansion dxable-args) x
+      (%defun name (fdefinition name) inline-expansion dxable-args)))
 
-  ;; KLUDGE: Why are fixups mixed up with toplevel forms? Couldn't
-  ;; fixups be done separately? Wouldn't that be clearer and better?
-  ;; -- WHN 19991204
-  (/show0 "doing cold toplevel forms and fixups")
   (unless (!c-runtime-noinform-p)
     (write `("Length(TLFs)=" ,(length *!cold-toplevels*)) :escape nil)
     (terpri))
   ;; only the basic external formats are present at this point.
-  (setq sb!impl::*default-external-format* :latin-1)
+  (setq sb-impl::*default-external-format* :latin-1)
 
-  (!with-init-wrappers
-    (loop for index-in-cold-toplevels from 0
-          for toplevel-thing in (prog1 *!cold-toplevels*
+  (loop with *package* = *package* ; rebind to self, as if by LOAD
+        for index-in-cold-toplevels from 0
+        for toplevel-thing in (prog1 *!cold-toplevels*
                                  (makunbound '*!cold-toplevels*))
         do
       #!+sb-show
@@ -202,20 +183,20 @@
             (setf (svref *!load-time-values* (third toplevel-thing))
                   (funcall (second toplevel-thing))))
         ((cons (eql :load-time-value-fixup))
-            (setf (sap-ref-word (int-sap (get-lisp-obj-address (second toplevel-thing)))
-                                (third toplevel-thing))
-                  (get-lisp-obj-address
-                   (svref *!load-time-values* (fourth toplevel-thing)))))
+         (destructuring-bind (object index value) (cdr toplevel-thing)
+           (aver (typep object 'code-component))
+           (aver (eq (code-header-ref object index) (make-unbound-marker)))
+           (setf (code-header-ref object index) (svref *!load-time-values* value))))
         ((cons (eql defstruct))
-         (apply 'sb!kernel::%defstruct (cdr toplevel-thing)))
+         (apply 'sb-kernel::%defstruct (cdr toplevel-thing)))
         (t
-         (!cold-lose "bogus operation in *!COLD-TOPLEVELS*")))))
+         (!cold-lose "bogus operation in *!COLD-TOPLEVELS*"))))
   (/show0 "done with loop over cold toplevel forms and fixups")
 
   ;; Precise GC seems to think these symbols are live during the final GC
   ;; which in turn enlivens a bunch of other "*!foo*" symbols.
   ;; Setting them to NIL helps a little bit.
-  (setq *!cold-defuns* nil *!cold-defconstants* nil *!cold-toplevels* nil)
+  (setq *!cold-defuns* nil *!cold-defsymbols* nil *!cold-toplevels* nil)
 
   ;; Now that L-T-V forms have executed, the symbol output chooser works.
   (setf (symbol-function 'choose-symbol-out-fun) real-choose-symbol-out-fun)
@@ -240,6 +221,7 @@
   (show-and-call !pathname-cold-init)
 
   (show-and-call stream-cold-init-or-reset)
+  (/show "Enabled buffered streams")
   (show-and-call !loader-cold-init)
   (show-and-call !foreign-cold-init)
   #!-(and win32 (not sb-thread))
@@ -251,7 +233,7 @@
 
   ;; Install closures as guards on some early PRINT-OBJECT methods so that
   ;; THREAD and RESTART print nicely prior to the real methods being installed.
-  (dovector (method (cdr (assoc 'print-object sb!pcl::*!trivial-methods*)))
+  (dovector (method (cdr (assoc 'print-object sb-pcl::*!trivial-methods*)))
     (unless (car method)
       (let ((classoid (find-classoid (third method))))
         (rplaca method
@@ -269,30 +251,28 @@
     ;; which would result in an error.
     (setf *standard-readtable* *readtable*))
   (setf *readtable* (copy-readtable *standard-readtable*))
-  (setf sb!debug:*debug-readtable* (copy-readtable *standard-readtable*))
-  (sb!pretty:!pprint-cold-init)
+  (setf sb-debug:*debug-readtable* (copy-readtable *standard-readtable*))
+  (sb-pretty:!pprint-cold-init)
   (setq *print-level* nil *print-length* nil) ; restore defaults
 
-  ;; the ANSI-specified initial value of *PACKAGE*
-  (setf *package* (find-package "COMMON-LISP-USER"))
-
   ;; Enable normal (post-cold-init) behavior of INFINITE-ERROR-PROTECT.
-  (setf sb!kernel::*maximum-error-depth* 10)
+  (setf sb-kernel::*maximum-error-depth* 10)
   (/show0 "enabling internal errors")
   (setf (extern-alien "internal_errors_enabled" int) 1)
+  (setf (symbol-function '%failed-aver) real-failed-aver-fun)
 
-  (show-and-call sb!disassem::!compile-inst-printers)
+  (show-and-call sb-disassem::!compile-inst-printers)
 
   ;; Toggle some readonly bits
-  (dovector (sc sb!c:*backend-sc-numbers*)
+  (dovector (sc sb-c:*backend-sc-numbers*)
     (when sc
-      (logically-readonlyize (sb!c::sc-move-funs sc))
-      (logically-readonlyize (sb!c::sc-load-costs sc))
-      (logically-readonlyize (sb!c::sc-move-vops sc))
-      (logically-readonlyize (sb!c::sc-move-costs sc))))
+      (logically-readonlyize (sb-c::sc-move-funs sc))
+      (logically-readonlyize (sb-c::sc-load-costs sc))
+      (logically-readonlyize (sb-c::sc-move-vops sc))
+      (logically-readonlyize (sb-c::sc-move-costs sc))))
 
   ; hppa heap is segmented, lisp and c uses a stub to call eachother
-  #!+hpux (%primitive sb!vm::setup-return-from-lisp-stub)
+  #!+hpux (%primitive sb-vm::setup-return-from-lisp-stub)
   ;; The system is finally ready for GC.
   (/show0 "enabling GC")
   (setq *gc-inhibit* nil)
@@ -356,7 +336,7 @@ process to continue normally."
       (os-exit (or code 1) :abort t)
       (let ((code (or code 0)))
         (with-deadline (:seconds nil :override t)
-          (sb!thread:grab-mutex *exit-lock*))
+          (sb-thread:grab-mutex *exit-lock*))
         (setf *exit-in-process* code
               *exit-timeout* timeout)
         (throw '%end-of-the-world t)))
@@ -365,21 +345,21 @@ process to continue normally."
 ;;;; initialization functions
 
 (defun thread-init-or-reinit ()
-  (sb!thread::init-job-control)
-  (sb!thread::get-foreground))
+  (sb-thread::init-job-control)
+  (sb-thread::get-foreground))
 
 (defun reinit ()
   #!+win32
-  (setf sb!win32::*ansi-codepage* nil)
+  (setf sb-win32::*ansi-codepage* nil)
   (setf *default-external-format* nil)
-  (setf sb!alien::*default-c-string-external-format* nil)
+  (setf sb-alien::*default-c-string-external-format* nil)
   ;; WITHOUT-GCING implies WITHOUT-INTERRUPTS.
   (without-gcing
     (finalizers-reinit)
     ;; Create *CURRENT-THREAD* first, since initializing a stream calls
     ;; ALLOC-BUFFER which calls FINALIZE which acquires **FINALIZER-STORE-LOCK**
     ;; which needs a valid thread in order to grab a mutex.
-    (sb!thread::init-initial-thread)
+    (sb-thread::init-initial-thread)
     ;; Initialize streams first, so that any errors can be printed later
     (stream-reinit t)
     (os-cold-init-or-reinit)
@@ -393,8 +373,8 @@ process to continue normally."
   (time-reinit)
   ;; If the debugger was disabled in the saved core, we need to
   ;; re-disable ldb again.
-  (when (eq *invoke-debugger-hook* 'sb!debug::debugger-disabled-hook)
-    (sb!debug::disable-debugger))
+  (when (eq *invoke-debugger-hook* 'sb-debug::debugger-disabled-hook)
+    (sb-debug::disable-debugger))
   (call-hooks "initialization" *init-hooks*))
 
 ;;;; some support for any hapless wretches who end up debugging cold
@@ -406,7 +386,7 @@ process to continue normally."
 (defun hexstr (thing)
   (/noshow0 "entering HEXSTR")
   (let* ((addr (get-lisp-obj-address thing))
-         (nchars (* sb!vm:n-word-bytes 2))
+         (nchars (* sb-vm:n-word-bytes 2))
          (str (make-string (+ nchars 2) :element-type 'base-char)))
     (/noshow0 "ADDR and STR calculated")
     (setf (char str 0) #\0
@@ -449,6 +429,6 @@ process to continue normally."
     defenum defun-cached with-globaldb-name def!type def!struct
     .
     #!+sb-show ()
-    #!-sb-show (/hexstr /nohexstr /noshow /noshow0 /noxhow
-                /primitive-print /show /show0 /xhow))
-  sb!impl::*!removable-symbols*)
+    #!-sb-show (/hexstr /nohexstr /noshow /noshow0
+                /primitive-print /show /show0))
+  *!removable-symbols*)
