@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;;; reading variable length integers
 ;;;;
@@ -158,8 +158,8 @@
            (,shift 0)
            (,acc 0)
            (,prev 0))
-       #-sb-xc-host (declare (notinline sb!kernel:%ldb)) ; lp#1573398
-       (declare (type (mod ,sb!vm:n-word-bits) ,shift)
+       #-sb-xc-host (declare (notinline sb-kernel:%ldb)) ; lp#1573398
+       (declare (type (mod ,sb-vm:n-word-bits) ,shift)
                 (type word ,acc ,prev))
        (loop
         (let ((,byte (ldb (byte 8 ,bytepos) ,integer)))
@@ -186,7 +186,6 @@
 ;;; A somewhat bad (slow and not-very-squishy) compressor
 ;;; that gets between 15% and 20% space savings in debug blocks.
 ;;; Lengthy input may be compressible by as much as 3:1.
-#-sb-xc-host
 (declaim (ftype (sfunction ((simple-array (unsigned-byte 8) 1)) (simple-array (unsigned-byte 8) 1))
                 lz-compress))
 (defun lz-compress (input)
@@ -244,10 +243,10 @@
            (coerce output '(simple-array (unsigned-byte 8) (*)))
            #-sb-xc-host
            (%shrink-vector (%array-data output) (fill-pointer output))))
+      #+(or)
       (aver (equalp input (lz-decompress result)))
       result)))
 
-#-sb-xc-host
 (declaim (ftype (sfunction ((simple-array (unsigned-byte 8) 1)) (simple-array (unsigned-byte 8) 1))
                 lz-decompress))
 (defun lz-decompress (input)
