@@ -21,7 +21,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;;; Thread safety
 ;;;;
@@ -58,7 +58,7 @@
   (declare (function function))
   ;; FIXME: Since name conflicts can be signalled while holding the
   ;; mutex, user code can be run leading to lock ordering problems.
-  (sb!thread:with-recursive-lock (*package-graph-lock*)
+  (sb-thread:with-recursive-lock (*package-graph-lock*)
     (funcall function)))
 
 ;;; a map from package names to packages
@@ -67,7 +67,7 @@
 
 (defmacro with-package-names ((table-var &key) &body body)
   `(let ((,table-var *package-names*))
-     (sb!thread::with-recursive-system-lock ((info-env-mutex ,table-var))
+     (sb-thread::with-recursive-system-lock ((info-env-mutex ,table-var))
        ,@body)))
 
 (defmethod make-load-form ((p package) &optional environment)
@@ -156,7 +156,7 @@ of :INHERITED :EXTERNAL :INTERNAL."
               "(~D+~D)/~D [~@[~,3f words/sym,~]load=~,1f%]"
               n-live n-deleted n-cells
               (unless (zerop n-live)
-                (/ (* (1+ (/ sb!vm:n-word-bytes)) n-cells) n-live))
+                (/ (* (1+ (/ sb-vm:n-word-bytes)) n-cells) n-live))
               (* 100 (/ n-filled n-cells))))))
 
 ;;; the maximum load factor we allow in a package hashtable
@@ -305,10 +305,6 @@ error if any of PACKAGES is not a valid package designator."
        (unlock-package package)))))
 
 (defun package-lock-violation-p (package &optional (symbol nil symbolp))
-  ;; KLUDGE: (package-lock package) needs to be before
-  ;; comparison to *package*, since during cold init this gets
-  ;; called before *package* is bound -- but no package should
-  ;; be locked at that point.
   (and package
        (package-lock package)
        ;; In package or implementation package
@@ -323,16 +319,16 @@ error if any of PACKAGES is not a valid package designator."
 
 (defun lexically-unlocked-symbol-p (symbol)
   (member symbol
-          (if (boundp 'sb!c::*lexenv*)
-              (let ((list (sb!c::lexenv-disabled-package-locks sb!c::*lexenv*)))
+          (if (boundp 'sb-c::*lexenv*)
+              (let ((list (sb-c::lexenv-disabled-package-locks sb-c::*lexenv*)))
                 ;; The so-called LIST might be an interpreter env.
                 #!+sb-fasteval
                 (unless (listp list)
                   (return-from lexically-unlocked-symbol-p
-                    (sb!interpreter::lexically-unlocked-symbol-p
+                    (sb-interpreter::lexically-unlocked-symbol-p
                      symbol list)))
                 list)
-              sb!c::*disabled-package-locks*)))
+              sb-c::*disabled-package-locks*)))
 
 ;;;; more package-locking these are NOPs unless :sb-package-locks is
 ;;;; in target features. Cross-compiler NOPs for these are in cross-misc.
@@ -373,7 +369,7 @@ error if any of PACKAGES is not a valid package designator."
                    (list
                     (return-from assert-symbol-home-package-unlocked
                       name))))
-         (package (symbol-package symbol)))
+         (package (sb-xc:symbol-package symbol)))
     (when (package-lock-violation-p package symbol)
       (package-lock-violation package
                               :symbol symbol
@@ -584,8 +580,8 @@ Experimental: interface subject to change."
 (define-condition bootstrap-package-not-found (condition)
   ((name :initarg :name :reader bootstrap-package-name)))
 (defun debootstrap-package (&optional condition)
-  (invoke-restart
-   (find-restart-or-control-error 'debootstrap-package condition)))
+  (declare (ignore condition))
+  (bug "No such thing as DEBOOTSTRAP-PACKAGE"))
 
 (defun find-package (package-designator)
   "If PACKAGE-DESIGNATOR is a package, it is returned. Otherwise PACKAGE-DESIGNATOR
@@ -632,18 +628,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
                   (nicknamed (when nicknames
                                (cdr (assoc string nicknames :test #'string=))))
                   (packageoid (or nicknamed (%get-package string *package-names*))))
-             (if (and (null packageoid)
-                      ;; FIXME: should never need 'debootstrap' hack
-                      (let ((mismatch (mismatch "SB!" string)))
-                        (and mismatch (= mismatch 3))))
-                 (restart-case
-                     (signal 'bootstrap-package-not-found :name string)
-                   (debootstrap-package ()
-                     (if (string= string "SB!XC")
-                         *cl-package*
-                         (find-package
-                          (substitute #\- #\! string :count 1)))))
-                 packageoid))))
+             packageoid)))
     ;; Is there a fundamental reason we don't declare the FTYPE
     ;; of FIND-PACKAGE-USING-PACKAGE letting the compiler do the checking?
     (t (error 'type-error
@@ -1286,7 +1271,7 @@ implementation it is ~S." *!default-package-use-list*)
                  (if presentp
                      (if (eq package-symbol chosen-symbol)
                          (shadow (list package-symbol) package) ; CLHS 11.1.1.2.5
-                         (if (eq (symbol-package package-symbol) package)
+                         (if (eq (sb-xc:symbol-package package-symbol) package)
                              (unintern package-symbol package) ; CLHS 11.1.1.2.5
                              (shadowing-import (list chosen-symbol) package)))
                      (shadowing-import (list chosen-symbol) package)))
@@ -1331,7 +1316,7 @@ uninterned."
                                   (package-internal-symbols package)
                                   (package-external-symbols package))
                               symbol)
-                 (if (eq (symbol-package symbol) package)
+                 (if (eq (sb-xc:symbol-package symbol) package)
                      (%set-symbol-package symbol nil))
                  t)
                 (t nil)))))))
@@ -1442,7 +1427,7 @@ the importation, then a correctable error is signalled."
   (with-package-graph ()
     (let* ((package (find-undeleted-package-or-lose package))
            (symbols (symbol-listify symbols))
-           (homeless (remove-if #'symbol-package symbols))
+           (homeless (remove-if #'sb-xc:symbol-package symbols))
            (syms ()))
       (with-single-package-locked-error ()
         (dolist (sym symbols)
@@ -1640,7 +1625,7 @@ PACKAGE."
             (result nil))
         (do-symbols (symbol package)
           (when (and (or (not external-only)
-                         (and (eq (symbol-package symbol) package)
+                         (and (eq (sb-xc:symbol-package symbol) package)
                               (eq (nth-value 1 (find-symbol (symbol-name symbol)
                                                             package))
                                   :external)))
@@ -1676,7 +1661,7 @@ PACKAGE."
 (defvar *!initial-symbols*)
 
 (defun !package-cold-init ()
-  (setf *package-graph-lock* (sb!thread:make-mutex :name "Package Graph Lock")
+  (setf *package-graph-lock* (sb-thread:make-mutex :name "Package Graph Lock")
         *package-names*
         (make-info-hashtable
          :comparator (named-lambda "PKG-NAME=" (a b)
@@ -1701,8 +1686,7 @@ PACKAGE."
               (info-gethash (package-%name pkg) names) pkg)
         (dolist (nick (package-%nicknames pkg))
           (setf (info-gethash nick names) (list pkg)))
-        (setf (package-lock pkg) nil
-              (package-%implementation-packages pkg) nil))))
+        (setf (package-%implementation-packages pkg) nil))))
 
   ;; pass 2 - set the 'tables' slots only after all tables have been made
   (dolist (spec *!initial-symbols*)
@@ -1791,7 +1775,7 @@ PACKAGE."
                                                (logand start-state 3))))))
                      (when (zerop index)
                        (return (advance start-state))))))
-          (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+          (declare (optimize (sb-c::insert-array-bounds-checks 0)))
           (if (logtest start-state +package-iter-check-shadows+)
               (let ((shadows (package-%shadowing-symbols (this-package))))
                 (scan (not (member sym shadows :test #'string=))))
@@ -1807,7 +1791,7 @@ PACKAGE."
                        ;; The warning is enough. It's ugly that both happen.
                        (warn "Compile-time package lock violation:~%  ~A"
                              condition)
-                       (sb!c:compiler-error condition))
+                       (sb-c:compiler-error condition))
                       (:eval
                        (eval-error condition))))))
     (with-single-package-locked-error (:symbol symbol control))))
