@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defconstant +number-stack-alignment-mask+ (1- (* n-word-bytes 2)))
 
@@ -223,18 +223,18 @@
 
 ;;; long-long support
 ;; (deftransform %alien-funcall ((function type &rest args) * * :node node)
-;;   (aver (sb!c::constant-lvar-p type))
-;;   (let* ((type (sb!c::lvar-value type))
-;;          (env (sb!c::node-lexenv node))
+;;   (aver (sb-c::constant-lvar-p type))
+;;   (let* ((type (sb-c::lvar-value type))
+;;          (env (sb-c::node-lexenv node))
 ;;          (arg-types (alien-fun-type-arg-types type))
 ;;          (result-type (alien-fun-type-result-type type)))
 ;;     (aver (= (length arg-types) (length args)))
 ;;     (if (or (some (lambda (type)
 ;;                     (and (alien-integer-type-p type)
-;;                          (> (sb!alien::alien-integer-type-bits type) 64)))
+;;                          (> (sb-alien::alien-integer-type-bits type) 64)))
 ;;                   arg-types)
 ;;             (and (alien-integer-type-p result-type)
-;;                  (> (sb!alien::alien-integer-type-bits result-type) 64)))
+;;                  (> (sb-alien::alien-integer-type-bits result-type) 64)))
 ;;         (collect ((new-args) (lambda-vars) (new-arg-types))
 ;;                  (loop with i = 0
 ;;                        for type in arg-types
@@ -242,7 +242,7 @@
 ;;                        do
 ;;                        (lambda-vars arg)
 ;;                        (cond ((and (alien-integer-type-p type)
-;;                                    (> (sb!alien::alien-integer-type-bits type) 64))
+;;                                    (> (sb-alien::alien-integer-type-bits type) 64))
 ;;                               (when (oddp i)
 ;;                                 ;; long-long is only passed in pairs of r0-r1 and r2-r3,
 ;;                                 ;; and the stack is double-word aligned
@@ -264,9 +264,9 @@
 ;;                               (new-args arg)
 ;;                               (new-arg-types type))))
 ;;                  (cond ((and (alien-integer-type-p result-type)
-;;                              (> (sb!alien::alien-integer-type-bits result-type) 64))
+;;                              (> (sb-alien::alien-integer-type-bits result-type) 64))
 ;;                         (let ((new-result-type
-;;                                 (let ((sb!alien::*values-type-okay* t))
+;;                                 (let ((sb-alien::*values-type-okay* t))
 ;;                                   (parse-alien-type
 ;;                                    (if (alien-integer-type-signed result-type)
 ;;                                        '(values (unsigned 64) (signed 64))
@@ -289,14 +289,14 @@
 ;;                                               :arg-types (new-arg-types)
 ;;                                               :result-type result-type)
 ;;                                            ,@(new-args))))))
-;;         (sb!c::give-up-ir1-transform))))
+;;         (sb-c::give-up-ir1-transform))))
 
 ;;; Callback
 #-sb-xc-host
 (defun alien-callback-accessor-form (type sap offset)
   (let ((parsed-type type))
     (if (alien-integer-type-p parsed-type)
-        (let ((bits (sb!alien::alien-integer-type-bits parsed-type)))
+        (let ((bits (sb-alien::alien-integer-type-bits parsed-type)))
                (let ((byte-offset
                       (cond ((< bits n-word-bits)
                              (- n-word-bytes
@@ -410,7 +410,7 @@
       (finalize-segment segment)
       ;; Now that the segment is done, convert it to a static
       ;; vector we can point foreign code to.
-      (let* ((buffer (sb!assem::segment-buffer segment))
+      (let* ((buffer (sb-assem::segment-buffer segment))
              (vector (make-static-vector (length buffer)
                                          :element-type '(unsigned-byte 8)
                                          :initial-contents buffer))
