@@ -46,7 +46,7 @@
 ;;; fail to infer (< I FIXNUM): it does not understand that this
 ;;; constraint follows from (TYPEP I (INTEGER 0 0)).
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 ;;; *CONSTRAINT-UNIVERSE* gets bound in IR1-PHASES to a fresh,
 ;;; zero-length, non-zero-total-size vector-with-fill-pointer.
@@ -300,7 +300,7 @@
   (etypecase y
     (ctype
        (awhen (lambda-var-ctype-constraints x)
-         (dolist (con (gethash (sb!kernel::type-class-info y) it) nil)
+         (dolist (con (gethash (sb-kernel::type-class-info y) it) nil)
            (when (and (eq (constraint-kind con) kind)
                       (eq (constraint-not-p con) not-p)
                       (type= (constraint-y con) y))
@@ -345,7 +345,7 @@
       (ctype
        (let ((index (ensure-hash (lambda-var-ctype-constraints x)))
              (vec   (ensure-vec  (lambda-var-inheritable-constraints x))))
-         (push con (gethash (sb!kernel::type-class-info y) index))
+         (push con (gethash (sb-kernel::type-class-info y) index))
          (vector-push-extend con vec)))
       (lvar
        (let ((index (ensure-hash (lambda-var-eq-constraints x))))
@@ -969,13 +969,26 @@
            (setf (node-derived-type ref) *wild-type*)
            (change-ref-leaf ref (find-constant t)))
           (t
-           (setf not-res
-                 (type-union not-res (make-member-type not-set not-fpz)))
-           (derive-node-type ref
-                             (make-single-value-type
-                              (or (type-difference res not-res)
-                                  res)))
-           (maybe-terminate-block ref nil))))
+           (let ((type (type-difference res
+                                        (type-union not-res
+                                                    (make-member-type not-set not-fpz)))))
+             ;; CHANGE-CLASS can change the type, lower down to standard-object,
+             ;; type propagation for classes is not as important anyway.
+             (cond #-sb-xc-host
+                   ((and
+                     (eq sb-pcl::**boot-state** 'sb-pcl::complete)
+                     (block nil
+                       (let ((standard-object (find-classoid 'standard-object)))
+                         (sb-kernel::map-type
+                          (lambda (type)
+                            (when (and (classoid-p type)
+                                       (csubtypep type standard-object))
+                              (return t)))
+                          type)))))
+                   (t
+                    (derive-node-type ref
+                                      (make-single-value-type type))
+                    (maybe-terminate-block ref nil)))))))
   (values))
 
 ;;;; Flow analysis
@@ -1219,11 +1232,12 @@
           (rest-of-blocks ())
           (seen-loop-p ()))
       (do-blocks (block component)
-        (when (and (not seen-loop-p) (loopy-p block))
-          (setq seen-loop-p t))
-        (if seen-loop-p
-            (push block rest-of-blocks)
-            (push block leading-blocks)))
+        (when (block-type-check block)
+          (when (and (not seen-loop-p) (loopy-p block))
+            (setq seen-loop-p t))
+          (if seen-loop-p
+              (push block rest-of-blocks)
+              (push block leading-blocks))))
       (values (nreverse leading-blocks) (nreverse rest-of-blocks)))))
 
 ;;; Append OBJ to the end of LIST as if by NCONC but only if it is not
@@ -1243,7 +1257,8 @@
   (let ((blocks-to-process ()))
     (flet ((enqueue (blocks)
              (dolist (block blocks)
-               (setq blocks-to-process (nconc-new block blocks-to-process)))))
+               (when (block-type-check block)
+                 (setq blocks-to-process (nconc-new block blocks-to-process))))))
       (multiple-value-bind (leading-blocks rest-of-blocks)
           (leading-component-blocks component)
         ;; Update every block once to account for changes in the
