@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 #-c-headers-only
 (macrolet ((frob ()
@@ -82,6 +82,7 @@
     (no-tls-value-marker "immediate")
 
     #!+sb-simd-pack (simd-pack "unboxed")
+    #!+sb-simd-pack-256 (simd-pack-256 "unboxed")
     (filler "unboxed")
 
     (simple-array "boxed")
@@ -187,7 +188,7 @@ static inline lispobj compute_lispobj(lispobj* base_addr) {
                                    (#.fun-pointer-lowtag      "fun")
                                    (#.other-pointer-lowtag    "other"))))
                (when pointer-kind
-                 (setf (svref ptrtab (ldb (byte 2 (- sb!vm:n-lowtag-bits 2)) i))
+                 (setf (svref ptrtab (ldb (byte 2 (- sb-vm:n-lowtag-bits 2)) i))
                        pointer-kind)
                  (setf (svref scavtab i) (format nil "~A_pointer" pointer-kind)
                        (svref sizetab i) "pointer"))))))
