@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (macrolet ((ea-for-xf-desc (tn slot)
              `(ea (- (* ,slot n-word-bytes) other-pointer-lowtag) ,tn)))
@@ -40,9 +40,7 @@
 
 ;;; complex float stack EAs
 (macrolet ((ea-for-cxf-stack (tn kind slot &optional base)
-             `(make-ea
-               :qword :base ,base
-               :disp (frame-byte-offset
+             `(ea (frame-byte-offset
                       (+ (tn-offset ,tn)
                        (cond ((= (tn-offset ,base) rsp-offset)
                               sp->fp-offset)
@@ -55,7 +53,8 @@
                          (:double
                             (ecase ,slot
                               (:real 1)
-                              (:imag 0)))))))))
+                              (:imag 0))))))
+                  ,base)))
   (defun ea-for-csf-data-stack (tn &optional (base rbp-tn))
     (ea-for-cxf-stack tn :single :real base))
   (defun ea-for-csf-real-stack (tn &optional (base rbp-tn))
@@ -897,7 +896,7 @@
                             :target mask))
                   (:arg-types ,type ,type)
                   (:temporary (:sc ,sc :from :eval) mask)
-                  (:temporary (:sc dword-reg) bits)
+                  (:temporary (:sc unsigned-reg) bits)
                   (:conditional :e)
                   (:generator ,cost
                     (when (location= y mask)
@@ -907,8 +906,7 @@
                       (setf y (register-inline-constant :aligned (tn-value y))))
                     (inst pcmpeqd mask y)
                     (inst movmskps bits mask)
-                    (inst cmp (if (location= bits eax-tn) al-tn bits)
-                          #b1111)))))
+                    (inst cmp :byte bits #b1111)))))
   (define-float-eql eql/single-float 4
     single-reg fp-single-immediate single-float)
   (define-float-eql eql/double-float 4
@@ -956,8 +954,7 @@
   (:vop-var vop)
   (:generator 3
     (when (or (location= y xmm)
-              (and (not (xmm-register-p x))
-                   (xmm-register-p y)))
+              (and (not (xmm-tn-p x)) (xmm-tn-p y)))
       (rotatef x y))
     (sc-case x
       (single-reg (setf xmm x))
@@ -989,8 +986,7 @@
   (:vop-var vop)
   (:generator 3
     (when (or (location= y xmm)
-              (and (not (xmm-register-p x))
-                   (xmm-register-p y)))
+              (and (not (xmm-tn-p x)) (xmm-tn-p y)))
       (rotatef x y))
     (sc-case x
       (double-reg
@@ -1028,7 +1024,7 @@
                               :load-if (not (sc-is y ,complex-constant-sc))))
                     (:arg-types ,complex-type ,complex-type)
                     (:temporary (:sc ,complex-sc :from :eval) cmp)
-                    (:temporary (:sc dword-reg) bits)
+                    (:temporary (:sc unsigned-reg) bits)
                     (:info)
                     (:conditional :e)
                     (:generator 3
@@ -1048,8 +1044,7 @@
                         (setf y (register-inline-constant :aligned (tn-value y))))
                       (inst ,cmp-inst :eq cmp y)
                       (inst ,mask-inst bits cmp)
-                      (inst cmp (if (location= bits eax-tn) al-tn bits)
-                            ,mask)))
+                      (inst cmp :byte bits ,mask)))
                   (define-vop (,complex-real-name ,complex-complex-name)
                     (:args (x :scs (,complex-sc ,complex-constant-sc)
                               :target cmp
@@ -1299,6 +1294,55 @@
         (move bits float)
         (inst sar bits 32)))))
 
+(define-vop (single-float-sign)
+  (:translate single-float-sign)
+  (:args (float :scs (descriptor-reg)))
+  (:arg-types single-float)
+  (:results (res :scs (descriptor-reg)))
+  (:policy :fast-safe)
+  (:generator 3
+    (move res float)
+    ;; preserve only the sign bit and widetag
+    (inst and res (constantize (ash #x80000000 32)))
+    ;; set the bits of corresponding to 1.0f0
+    (inst or  res (constantize (logior (ash #x3F800000 32)
+                                       single-float-widetag)))))
+;;; Return Y with its sign changed to that of X.
+;;; The arguments match FLOAT-SIGN which is the opposite of C's copysign().
+(define-vop (single-float-copysign)
+  (:translate single-float-copysign)
+  (:args (x :scs (descriptor-reg))
+         (y :scs (descriptor-reg)))
+  (:arg-types single-float single-float)
+  (:results (res :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg :from (:argument 0)) temp)
+  (:policy :fast-safe)
+  (:generator 3
+    (move temp x)
+    (move res y)
+    (inst shl res 1)   ; discard result's sign bit
+    (inst shl temp 1)  ; copy X's sign bit into the carry flag
+    (inst rcr res 1))) ; rotate carry into the result
+
+(define-vop (double-float-bits)
+  (:args (float :scs (double-reg descriptor-reg)
+                :load-if (not (sc-is float double-stack))))
+  (:results (bits :scs (signed-reg)))
+  (:arg-types double-float)
+  (:result-types signed-num)
+  (:translate double-float-bits)
+  (:policy :fast-safe)
+  (:generator 5
+     (sc-case float
+       (double-reg
+        (inst movq bits float))
+       (double-stack
+        (inst mov bits (ea (frame-byte-offset (tn-offset float)) rbp-tn)))
+       (descriptor-reg
+        (inst mov bits (ea (- (ash double-float-value-slot word-shift)
+                              other-pointer-lowtag)
+                           float))))))
+
 (define-vop (double-float-high-bits)
   (:args (float :scs (double-reg descriptor-reg)
                 :load-if (not (sc-is float double-stack))))
@@ -1307,7 +1351,6 @@
   (:result-types signed-num)
   (:translate double-float-high-bits)
   (:policy :fast-safe)
-  (:vop-var vop)
   (:generator 5
      (sc-case float
        (double-reg
@@ -1330,7 +1373,6 @@
   (:result-types unsigned-num)
   (:translate double-float-low-bits)
   (:policy :fast-safe)
-  (:vop-var vop)
   (:generator 5
      (sc-case float
         (double-reg
@@ -1507,7 +1549,7 @@
 (defknown swap-complex ((complex float)) (complex float)
     (foldable flushable movable always-translatable))
 (defoptimizer (swap-complex derive-type) ((x))
-  (sb!c::lvar-type x))
+  (sb-c::lvar-type x))
 (defun swap-complex (x)
   (complex (imagpart x) (realpart x)))
 (define-vop (swap-complex-single-float)
