@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; instruction-like macros
 
@@ -26,14 +26,14 @@
   ;; all seemingly redundant moves, and then eliminate them before emission.
   ;; This way we can track movement of TNs into the same physical reg in a
   ;; different SC which will give useful information to a peephole optimizer.
-  (when (and (sb!x86-64-asm::register-p dst)
-             (sb!x86-64-asm::register-p src))
-    (let ((dst (sb!x86-64-asm::reg-id dst))
-          (src (sb!x86-64-asm::reg-id src)))
-      (aver (sb!x86-64-asm::is-gpr-id-p dst))
-      (aver (sb!x86-64-asm::is-gpr-id-p src))
-      (unless (= (sb!x86-64-asm::reg-id-num dst)
-                 (sb!x86-64-asm::reg-id-num src))
+  (when (and (sb-x86-64-asm::register-p dst)
+             (sb-x86-64-asm::register-p src))
+    (let ((dst (sb-x86-64-asm::reg-id dst))
+          (src (sb-x86-64-asm::reg-id src)))
+      (aver (sb-x86-64-asm::is-gpr-id-p dst))
+      (aver (sb-x86-64-asm::is-gpr-id-p src))
+      (unless (= (sb-x86-64-asm::reg-id-num dst)
+                 (sb-x86-64-asm::reg-id-num src))
         (inst mov dst src)))
     (return-from move))
   (unless (location= dst src)
@@ -52,6 +52,14 @@
       ((single-sse-reg double-sse-reg)
        (aver (xmm-tn-p src))
        (inst movaps dst src))
+      #!+sb-simd-pack-256
+      ((int-avx2-reg avx2-reg)
+       (aver (xmm-tn-p src))
+       (inst vmovdqa dst src))
+      #!+sb-simd-pack-256
+      ((single-avx2-reg double-avx2-reg)
+       (aver (xmm-tn-p src))
+       (inst vmovaps dst src))
       (t
        (inst mov dst src)))))
 
@@ -341,7 +349,8 @@
                `((gen-cell-set
                    (ea (- (* ,offset n-word-bytes) ,lowtag)
                        object index (ash 1 (- word-shift n-fixnum-tag-bits)))
-                   value result)))))
+                   value result vop
+                   ,(eq name 'set-funcallable-instance-info))))))
      ,@(when want-both-variants
          `((define-vop (,(symbolicate name "-C"))
             ,@(when translate
@@ -356,11 +365,13 @@
                         ,el-type)
             (:results (result :scs ,scs))
             (:result-types ,el-type)
+            (:vop-var vop)
             (:generator 3                    ; was 5
               (gen-cell-set
                    (ea (- (* (+ ,offset index) n-word-bytes) ,lowtag)
                        object)
-                   value result))))))))
+                   value result vop
+                   ,(eq name 'set-funcallable-instance-info)))))))))
 
 (defmacro define-full-setter+offset (name type offset lowtag scs el-type &optional translate)
   `(progn
