@@ -92,7 +92,7 @@
         (let ((sexpr (make-symbol "SEXPR")))
           (setq vars (nconc (subseq vars 0 1) (list sexpr) aux-vars)
                 ignore `((ignore ,sexpr))))))
-    (setq name (cons 'eval (if (listp name) name (list name))))
+    (setq name (cons '.eval. (if (listp name) name (list name))))
     `(handler ,(if captured-vars
                    (if (= (length captured-vars) 1)
                        `(named-lambda ,name (,(first captured-vars) ,@vars)
@@ -242,11 +242,11 @@
                              collect
                              `(list ,@(loop for j below n-cells
                                             collect `(access ,depth ,j))))))))
-  (defparameter *fast-lexvar-reffers*
+  (define-load-time-global *fast-lexvar-reffers*
     (macrolet ((access (up across)
                  `(hlambda GET-VAR () (env) (ref ,up ,across))))
       (array-of 4 10))) ; for 4 scopes and 10 names per scope you can go fast
-  (defparameter *fast-lexvar-setters* ; These are lambdas, not handlers
+  (define-load-time-global *fast-lexvar-setters* ; These are lambdas, not handlers
     (macrolet ((access (up across)
                  `(named-lambda (eval SET-VAR) (form env sexpr)
                     (declare #.+handler-optimize+ (ignore sexpr))
@@ -437,80 +437,79 @@
 ;;; ENV is for package-lock checks and also to make a quick guess about
 ;;; whether a block can be created at the same time as the variable bindings.
 (defun digest-lambda (env proto-fn)
-  (multiple-value-bind (required-args other-args keyword-bits decoder n-opt)
-      (encode-lambda-bindings (proto-fn-lambda-list proto-fn)
-                              (lambda (x) (%sexpr (second x))))
-    (let* ((decls (proto-fn-decls proto-fn))
-           (declared-specials (declared-specials decls))
-           (free-specials
-            (remove-if (lambda (sym)
-                         (or (memq sym required-args) (memq sym other-args)))
-                       declared-specials))
-           (n-required (length required-args))
-           (n-other (length other-args))
-           (n-lambda-vars (+ n-required n-other))
-           (symbols (make-array (+ n-lambda-vars (length free-specials)))))
-      ;; Fill in the SYMBOLS
-      (replace symbols required-args)
-      (replace symbols other-args :start1 n-required)
-      (replace symbols free-specials :start1 n-lambda-vars)
-      (dotimes (i (length symbols))
-        (setf (svref symbols i)
-              (cons (svref symbols i)
-                    (and (< i n-lambda-vars) *universal-type*))))
-      (multiple-value-bind (block-name forms block-share-env-p)
-          (extract-lambda-block (proto-fn-forms proto-fn) decoder env)
-        (let* ((special-b
-                (mark-bound-specials env declared-specials
-                                     symbols n-lambda-vars))
-               (required-mask (lognot (ash -1 n-required)))
-               (required-specials ; one list for PROGV in WITH-LET-BINDINGS
-                (nreverse ; WITH-LET-BINDINGS uses PUSH to accumulate values
-                 (collect-progv-symbols
-                  symbols n-lambda-vars (logand special-b required-mask))))
-               (other-specials ; one list per PROGV in WITH-LET*-BINDINGS.
-                (mapcar #'list (collect-progv-symbols
-                                symbols n-lambda-vars
-                                (logandc2 special-b required-mask))))
-               (frame
-                (make-lambda-frame
-                 :min-args n-required :n-optional n-opt
-                 :n-bound-vars n-lambda-vars :keyword-bits keyword-bits
-                 :symbols symbols :values (or decoder #())
-                 :special-b special-b
-                 :specials (if required-specials
-                               (cons required-specials other-specials)
-                               other-specials)
-                 :declarations decls
-                 :%old-policy (env-policy env)
-                 :%policy (new-policy env decls)
-                 :block-name block-name :share-block-p block-share-env-p
-                 :sexpr (%progn forms))))
-          (process-typedecls frame env symbols)
-          (setf (proto-fn-%frame proto-fn) frame
-                (proto-fn-cookie proto-fn) *globaldb-cookie*)
-          (values frame *globaldb-cookie*))))))
+  (binding*
+      (((required-args other-args keyword-bits decoder n-opt)
+        (encode-lambda-bindings (proto-fn-lambda-list proto-fn)
+                                (lambda (x) (%sexpr (second x)))))
+       (decls (proto-fn-decls proto-fn))
+       (n-required (length required-args))
+       (n-other (length other-args))
+       (n-lambda-vars (+ n-required n-other))
+       (declared-specials (declared-specials decls))
+       (free-specials
+        (remove-if (lambda (sym)
+                     (or (memq sym required-args) (memq sym other-args)))
+                   declared-specials))
+       (symbols
+        (let ((a (make-array (+ n-lambda-vars (length free-specials)))))
+          (replace a required-args)
+          (replace a other-args :start1 n-required)
+          (replace a free-specials :start1 n-lambda-vars)
+          (dotimes (i n-lambda-vars a) ; Unique-ify the binding cells.
+            (setf (svref a i) (list (svref a i))))))
+       (special-b
+        (mark-bound-specials env declared-specials symbols n-lambda-vars))
+       (required-mask (lognot (ash -1 n-required)))
+       (required-specials ; one list for PROGV in WITH-LET-BINDINGS
+        (nreverse ; WITH-LET-BINDINGS uses PUSH to accumulate values
+         (collect-progv-symbols
+          symbols n-lambda-vars (logand special-b required-mask))))
+       (other-specials ; one list per PROGV in WITH-LET*-BINDINGS.
+        (mapcar #'list
+                (collect-progv-symbols
+                 symbols n-lambda-vars (logandc2 special-b required-mask))))
+       ((block-name forms block-share-env-p)
+        (extract-lambda-block (proto-fn-forms proto-fn) decoder env))
+       (frame (make-lambda-frame
+               :min-args n-required :n-optional n-opt
+               :n-bound-vars n-lambda-vars :keyword-bits keyword-bits
+               :symbols symbols :values (or decoder #())
+               :special-b special-b
+               :specials (if required-specials
+                             (cons required-specials other-specials)
+                             other-specials)
+               :declarations decls :%policy (new-policy env decls)
+               :block-name block-name :share-block-p block-share-env-p
+               :sexpr (%progn forms))))
+    (process-typedecls frame env n-lambda-vars symbols)
+    (setf (proto-fn-%frame proto-fn) frame
+          (proto-fn-cookie proto-fn) *globaldb-cookie*)
+    (values frame *globaldb-cookie*)))
 
 (defun make-local-fn-scope (decls funs forms env)
   (let ((specials (free-specials env decls)))
     (process-typedecls (%make-local-fn-scope
                         decls (new-policy env decls)
                         funs (%progn forms) specials)
-                       env specials)))
+                       env 0 specials)))
 
 (defun err-too-few-args (fun n-args)
+  (declare (explicit-check))
   (let ((frame (interpreted-function-frame fun)))
-    (ip-error "~S received ~D argument~:P but expects~:[ at least~;~] ~D."
-           (name-for-fun fun) n-args
-           (eql (lambda-frame-min-args frame) (lambda-frame-max-args frame))
-           (lambda-frame-min-args frame))))
+    (%program-error "~S received ~D argument~:P but expects~:[ at least~;~] ~D."
+                    (name-for-fun fun) n-args
+                    (eql (lambda-frame-min-args frame)
+                         (lambda-frame-max-args frame))
+                    (lambda-frame-min-args frame))))
 
 (defun err-too-many-args (fun n-args)
+  (declare (explicit-check))
   (let ((frame (interpreted-function-frame fun)))
-    (ip-error "~S received ~D argument~:P but expects~:[ at most~;~] ~D"
-           (name-for-fun fun) n-args
-           (eql (lambda-frame-min-args frame) (lambda-frame-max-args frame))
-           (lambda-frame-max-args frame))))
+    (%program-error "~S received ~D argument~:P but expects~:[ at most~;~] ~D"
+                    (name-for-fun fun) n-args
+                    (eql (lambda-frame-min-args frame)
+                         (lambda-frame-max-args frame))
+                    (lambda-frame-max-args frame))))
 
 (defmacro with-lambda-frame ((frame-var fun if-invalid) &body if-valid)
   ;; If any global variable has changed its :KIND, this function's lambda
@@ -533,7 +532,7 @@
                (if (oddp n-more)
                    (fail-odd-length (+ n-seen n-more)))))
            (fail-odd-length (n)
-             (ip-error "odd number of &KEY arguments: ~D" n))
+             (%program-error "odd number of &KEY arguments: ~D" n))
            (fail-other-key ()
              (let ((n-allowed (keyword-bits-n-keys control-bits)) bad)
                (loop for (key val) on list by #'cddr ; rescan to collect them
@@ -541,10 +540,11 @@
                                 (find key allowed :end n-allowed))
                      do (pushnew key bad))
                (let ((plural (cdr bad)))
-                 (ip-error "Keyword~*~:[~;s~]~2:* ~{~S~^,~} ~:[is~;are~] not ~
-                            ~:[allowed.~;in the allowed set ~:*~S~]"
-                           (nreverse bad) plural
-                           (replace (make-list n-allowed) allowed))))))
+                 (%program-error "Keyword~*~:[~;s~]~2:* ~{~S~^,~} ~
+                                  ~:[is~;are~] not ~:[allowed.~;in the ~
+                                  allowed set ~:*~S~]"
+                                 (nreverse bad) plural
+                                 (replace (make-list n-allowed) allowed))))))
     (if (keyword-bits-allowp control-bits)
         (check-odd-length list 0)
         (let ((n-allowed (keyword-bits-n-keys control-bits))
@@ -580,9 +580,8 @@
 (defun interpreter-trampoline (fun &rest args)
   (setf (values (interpreted-function-frame fun)
                 (interpreted-function-cookie fun))
-        (proto-fn-frame (interpreted-function-proto-fn fun)
-                        (interpreted-function-env fun)))
-  (apply (setf (funcallable-instance-fun fun) (interpreted-applicator fun))
+        (proto-fn-frame (fun-proto-fn fun) (interpreted-function-env fun)))
+  (apply (setf (%funcallable-instance-fun fun) (interpreted-applicator fun))
          args))
 
 ;;; The most general case of interpreted function application.
@@ -704,14 +703,14 @@
        (invoke (n &aux (args (subseq '(arg1 arg2 arg3 arg4 arg5) 0 n)))
          `(if (eql (lambda-frame-block-name (interpreted-function-frame fun))
                    0)
-              (lambda ,args
+              (named-lambda (.apply. ,n) ,args
                 (declare #.+handler-optimize+ (optimize sb-c:verify-arg-count))
                 (with-lambda-frame
                     (frame fun (interpreter-trampoline fun ,@args))
                   (dispatch (lambda-frame-sexpr frame)
                             (new-env make-var-env
-                                     ,(if (zerop n) nil `(vector ,@args))))))
-              (lambda ,args
+                                     ,(if (zerop n) #() `(vector ,@args))))))
+              (named-lambda (.apply. ,n) ,args
                 (declare #.+handler-optimize+ (optimize sb-c:verify-arg-count))
                 (with-lambda-frame
                     (frame fun (interpreter-trampoline fun ,@args))
@@ -719,7 +718,7 @@
                     (catch exit
                       (dispatch (lambda-frame-sexpr frame)
                                 (new-env make-lambda-env
-                                         ,(if (zerop n) nil `(vector ,@args))
+                                         ,(if (zerop n) #() `(vector ,@args))
                                          exit)))))))))
     (case (lambda-frame-min-args (interpreted-function-frame fun))
       (5 (invoke 5))
@@ -740,13 +739,13 @@
                    (dotimes (i (lambda-frame-min-args frame) ,form)
                      (setf (svref cells i) (nth i args)))))))
          (if (eql (lambda-frame-block-name (interpreted-function-frame fun)) 0)
-             (lambda (&rest args)
+             (named-lambda (.apply.) (&rest args)
                (declare #.+handler-optimize+)
                (with-lambda-frame
                    (frame fun (apply #'interpreter-trampoline fun args))
                  (with-args (dispatch (lambda-frame-sexpr frame)
                                       (new-env make-var-env cells)))))
-             (lambda (&rest args)
+             (named-lambda (.apply.) (&rest args)
                (declare #.+handler-optimize+)
                (with-lambda-frame
                    (frame fun (apply #'interpreter-trampoline fun args))
@@ -761,7 +760,7 @@
 ;;; are never listified (it's an implicit &MORE arg). I don't want to write code
 ;;; to manually listify, nor hand-roll GETF. Also no special bindings permitted.
 (defun applicator/&optional (fun)
-  (lambda (&rest args)
+  (named-lambda (.apply. &optional) (&rest args)
     (declare #.+handler-optimize+)
     (with-lambda-frame (frame fun (apply #'interpreter-trampoline fun args))
       (let* ((n-actual (let* ((min (lambda-frame-min-args frame))
@@ -833,8 +832,7 @@
 ;;; and checks for a binding of *SELF-APPLYHOOK* on each call.
 (defun interpreter-hooked-trampoline (fun &rest args)
   (multiple-value-bind (frame cookie)
-      (proto-fn-frame (interpreted-function-proto-fn fun)
-                      (interpreted-function-env fun))
+      (proto-fn-frame (fun-proto-fn fun) (interpreted-function-env fun))
     (setf (values (interpreted-function-frame fun)
                   (interpreted-function-cookie fun)) (values frame cookie))
     ;; *SELF-APPLYHOOK* isn't the *APPLYHOOK* as described by CLtL.
@@ -879,18 +877,6 @@ Test case.
 (get-thinginator) ; errs - it correctly perceives the redefinition
 |#
 
-;;; Figure out if F is supposed to be funcallable, versus is the error trampoline
-;;; that is installed in a global macro's fdefn object.  It is very efficient
-;;; to do this by seeing if F is a closure over the error trampoline, versus asking
-;;; the horribly slow globaldb if F's name's macro-function is NIL.
-;;; In the common case where F isn't a closure, it definitely isn't a macro.
-(declaim (inline %looks-like-macro-p))
-(defun %looks-like-macro-p (f) ; f is a FUNCTION
-  (and (= (fun-subtype (truly-the function f)) sb-vm:closure-header-widetag)
-       ;; compare to a known global macro
-       (eq (load-time-value (%closure-fun (symbol-function 'or)) t)
-           (%closure-fun f))))
-
 ;;; Return T if SYMBOL might get redefined as a macro when it was previously
 ;;; a function and vice versa. Extra checks are done on every use of the symbol
 ;;; as a function name.
@@ -967,7 +953,7 @@ Test case.
 
 (defun arglist-to-sexprs (args)
   (let ((argc (or (list-length args)
-                  (ip-error "Malformed function call"))))
+                  (%program-error "Malformed function call"))))
     (values (mapcar #'%sexpr args) argc)))
 
 ;;; Return a handler which decides whether its supplied SEXPR needs
@@ -1113,7 +1099,7 @@ Test case.
     (if (allow-macro-redefinition env)
         (macrolet ((re-expand-p ()
                      '(let ((f (fdefn-fun fdefn)))
-                        (and f (%looks-like-macro-p f)))))
+                        (and f (sb-impl::macro/special-guard-fun-p f)))))
           (generate-switch))
         (macrolet ((re-expand-p () nil)) (generate-switch)))))
 
@@ -1174,16 +1160,32 @@ Test case.
       (let ((f (symbol-function fname)))
         (when (and (interpreted-function-p f)
                    (structure-instance-accessor-p fname))
-          ;: Compile the accessor. If it was defined in a non-null environment,
-          ;; conversion to a lexenv could say "too complex", so we want to
-          ;; force it. Passing two arguments to COMPILE achieves this.
+          ;: Compile the accessor using an explicit call to COMPILE with a
+          ;; lambda expression. Don't simply call (COMPILE FNAME) because
+          ;; if it was defined in a non-null environment, conversion to a
+          ;; lexenv could say "too complex". Additionally the debug source
+          ;; info would be misleading.
           ;; We can be confident that the expression doesn't need a lexenv,
           ;; because if the function were incompatible with the source-transform,
           ;; %DEFUN would have cleared the :source-transform, and fname would not
           ;; satisfy STRUCTURE-INSTANCE-ACCESSOR-P.
           #+nil (format t "~&; Interpreter: Compiling ~S~%" fname)
           ;; FIXME: ensure that the compiled function is safe.
-          (compile fname (function-lambda-expression f)))))
+          (let* ((compiled-fun (compile nil (function-lambda-expression f)))
+                 (code (fun-code-header compiled-fun))
+                 (cdi (%code-debug-info code))
+                 (source (sb-c::compiled-debug-info-source cdi)))
+            ;; Ensure that the debug-source-namestring of the compiled thing
+            ;; reflects the source file of the original interpreted function.
+            (setf (debug-source-namestring source)
+                  (sb-kernel::function-file-namestring f)
+                  (debug-source-created source) nil) ; = unknown
+            ;; Clobber the TLF-NUM + OFFSET. We should do better that that,
+            ;; but interpreted functions do not track their file offset,
+            ;; so the conservative thing is to plug in NILs, not bogus values
+            ;; from whatever file (if any) is currently being loaded.
+            (setf (sb-c::compiled-debug-info-tlf-num+offset cdi) 0)
+            (setf (symbol-function fname) compiled-fun)))))
 
     (when (fluid-def-p fname)
       ;; Return a handler that calls FNAME very carefully
@@ -1219,7 +1221,9 @@ Test case.
                (digest-local-call frame-ptr (cdr args))))))))
 
   (let ((n-args 0)
-        (fun (fdefinition fname)))
+        (fun (if (typep fname '(cons (eql sb-pcl::slot-accessor)))
+                 (funcall 'sb-pcl::ensure-accessor fname)
+                 (fdefinition fname))))
     (multiple-value-setq (args n-args) (arglist-to-sexprs args))
 
     ;; Fold if every arg when trivially constant and the function is foldable.
