@@ -14,10 +14,7 @@ arch_os_context_mxcsr_addr(os_context_t *context)
 
 typedef register_t os_context_register_t;
 
-static inline os_context_t *arch_os_get_context(void **void_context)
-{
-    return (os_context_t *) *void_context;
-}
+#include "arch-os-generic.inc"
 
 #if __DARWIN_UNIX03
 #define CONTEXT_ADDR_FROM_STEM(stem) (os_context_register_t*)&context->uc_mcontext->__ss.__##stem
@@ -27,5 +24,6 @@ static inline os_context_t *arch_os_get_context(void **void_context)
 
 #define RESTORE_FP_CONTROL_FROM_CONTEXT
 void os_restore_fp_control(os_context_t *context);
+void set_thread_stack(void *);
 
 #endif /* _X86_64_DARWIN_OS_H */
